@@ -64,6 +64,11 @@
 
 #include <libxfce4util/libxfce4util.h>
 
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+#define SN_API_NOT_YET_FROZEN 1
+#include <libsn/sn.h>
+#endif
+
 #include "mcs-channel.h"
 #include "mcs-client.h"
 
@@ -366,7 +371,7 @@ parse_settings (guchar      *data,
     buffer.pos = buffer.data = data;
     buffer.len = len;
 
-    result = fetch_card8 (&buffer, (gchar *) & buffer.byte_order);
+    result = fetch_card8 (&buffer, (CARD8 *) &buffer.byte_order);
     if (buffer.byte_order != MSBFirst && buffer.byte_order != LSBFirst)
     {
         fprintf (stderr, _("Invalid byte order in MCS property\n"));
@@ -426,7 +431,7 @@ parse_settings (guchar      *data,
         memcpy (setting->name, buffer.pos, name_len);
 
         setting->name[name_len] = '\0';
-        setting->channel_name = g_ascii_strup (channel_name, strlen (channel_name));
+        setting->channel_name = g_ascii_strup (channel_name, -1);
         buffer.pos += pad_len;
 
         result = fetch_card32 (&buffer, &v_int);
@@ -800,6 +805,37 @@ mcs_client_process_event (McsClient *client,
     return FALSE;
 }
 
+
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+static gchar *
+mcs_client_generate_snotify_message(Display *display,
+                                    int screen,
+                                    const gchar *message)
+{
+    SnDisplay *sndpy;
+    SnLauncheeContext *snctx;
+    gchar *new_message;
+    
+    sndpy = sn_display_new(display,
+                           (SnDisplayErrorTrapPush)sn_display_error_trap_push,
+                           (SnDisplayErrorTrapPop)sn_display_error_trap_pop);
+    
+    snctx = sn_launchee_context_new_from_environment(sndpy, screen);
+    if(snctx) {
+        new_message = g_strconcat(message, "|",
+                                  sn_launchee_context_get_startup_id(snctx),
+                                  NULL);
+        sn_launchee_context_unref(snctx);
+    } else
+        new_message = g_strdup(message);
+    
+    sn_display_unref(sndpy);
+    
+    return new_message;
+}
+#endif
+
+
 /**
  * mcs_client_show:
  * @display: The X display on which the MCS manager is running.
@@ -817,7 +853,7 @@ mcs_client_show (Display     *display,
     static Atom show_atom = None;
     Window w;
     Atom selection_atom = None;
-    gchar *buffer;
+    gchar *buffer, *message1;
 
     g_return_if_fail (message != NULL);
 
@@ -826,6 +862,11 @@ mcs_client_show (Display     *display,
     selection_atom = XInternAtom (display, buffer, FALSE);
     g_free (buffer);
     show_atom = XInternAtom (display, "SHOW", FALSE);
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+    message1 = mcs_client_generate_snotify_message(display, screen, message);
+#else
+    message1 = g_strdup(message);
+#endif
 
     w = XGetSelectionOwner (display, selection_atom);
     if (w != None)
@@ -837,6 +878,8 @@ mcs_client_show (Display     *display,
 
     XUngrabServer (display);
     XFlush (display);
+    
+    g_free (message1);
 }
 
 
